set(CMAKE_MODULE_PATH ${CMAKE_CURRENT_SOURCE_DIR}/../cmake)
include(${CMAKE_CURRENT_SOURCE_DIR}/../cmake/utility_macros.cmake)
include(${CMAKE_CURRENT_SOURCE_DIR}/../cmake/platform.cmake)
include(${CMAKE_CURRENT_SOURCE_DIR}/../cmake/default_flags.cmake)

if(EB_PLATFORM_OS_QNX)
    set(GTF_TARGET_SDK_ROOT "${CMAKE_CURRENT_SOURCE_DIR}/../../platform/qnx_6.6_armv7" CACHE PATH "Folder containing the GTF target SDK (e.g. ../../platform/qnx_6.6_armv7)")
elseif(EB_PLATFORM_OS_LINUX)
    set(GTF_TARGET_SDK_ROOT "${CMAKE_CURRENT_SOURCE_DIR}/../../platform/linux_2.4_armv7_vivante" CACHE PATH "Folder containing the GTF target SDK (e.g. ../../platform/linux_2.4_armv7_vivante or ../../platform/debian_8.2_i686)")
else()
    set(GTF_TARGET_SDK_ROOT "${CMAKE_CURRENT_SOURCE_DIR}/../../platform/win32" CACHE PATH "Folder containing the GTF target SDK (e.g. ../../platform/win32)")
endif()

include(${GTF_TARGET_SDK_ROOT}/lib/gtf-exports.cmake)
include_directories(${GTF_TARGET_SDK_ROOT}/include)

macro(gtf_add_sdk_example)
    gtf_parse_arguments(EB_TEMP
         "INCLUDE_PATH_VARS;INCLUDE_PATHS;LIB_PATH_VARS;LIB_VARS;SOURCES;LIBS;DEFINITIONS;FLAGS"
         ""
         ${ARGN}
         )

    eb_add_library(${PROJECT_NAME}
        SHARED
        INCLUDE_PATH_VARS
            ${EB_TEMP_INCLUDE_PATH_VARS}
        INCLUDE_PATHS
            ${EB_TEMP_INCLUDE_PATHS}
        LIB_PATH_VARS
            ${EB_TEMP_LIB_PATH_VARS}
        LIB_VARS
            ${EB_TEMP_LIB_VARS}
        SOURCES
            ${EB_TEMP_SOURCES}
        LIBS
            ${EB_TEMP_LIBS}
            GtfCore
        DEFINITIONS
            ${EB_TEMP_DEFINITIONS}
        FLAGS
            ${EB_TEMP_FLAGS}
    )

    set(${PROJECT_NAME}_INSTALL_DESTINTATION ${CMAKE_INSTALL_PREFIX}/${GTF_EXAMPLES_NAME}/${PROJECT_NAME}/bin)

    install(
        TARGETS
            ${PROJECT_NAME}
        RUNTIME DESTINATION
            ${${PROJECT_NAME}_INSTALL_DESTINTATION}
        LIBRARY DESTINATION
            ${${PROJECT_NAME}_INSTALL_DESTINTATION}
        )
endmacro()
