////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_WIDGET_CREPUSCULAR_WIDGET_H_INCLUDED
#define CUSTOM_WIDGET_CREPUSCULAR_WIDGET_H_INCLUDED

#include <gtf/widgetset/WidgetModel.h>
#include <gtf/widgetset/AbstractWidget.h>
#include <gtf/types/Noncopyable.h>

extern gtf::widgetset::AbstractWidget* createCrepuscularWidget(gtf::widgetset::WidgetContext const& context_, gtf::widgetset::WidgetModel& model_);
extern void destroyCrepuscularWidget(gtf::widgetset::AbstractWidget* widget_);

class CrepuscularWidget
    : public gtf::widgetset::AbstractWidget
    , public gtf::types::Noncopyable
    , public gtf::memory::Memory
{
public:

    //!
    //! \brief Constructor
    //!
    //! \param model_[in] The widget model.
    //! \param context_[in] The widget context.
    //!
    CrepuscularWidget(gtf::widgetset::WidgetModel& model_, gtf::widgetset::WidgetContext const& context_);

    //!
    //! \brief Destructor
    //!
    ~CrepuscularWidget();

    //!
    //! \brief Method called once in the widget's lifetime.
    //!
    //! \param context_[in] The widget context
    //!
    //! \details Possibly there are more calls if there are down transitions in the state machine
    //!          In this case this is a (re)init.
    //!
    virtual void init(gtf::widgetset::WidgetContext const& context_);

    //!
    //! \brief Method called once in the widget's lifetime.
    //!
    //! \param context_[in] The widget context
    //!
    //! \details It is called after the widget tree, virtual and link properties are initialized.
    //!
    virtual void prepare(gtf::widgetset::WidgetContext const& context_);

    //!
    //! \brief Method called after leaving the view.
    //!
    //! \param context_[in] The widget context.
    //!
    virtual void cleanup(gtf::widgetset::WidgetContext const& context_);
};

#endif // CREPUSCULAR_WIDGET_H_INCLUDED
