////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_WIDGET_CREPUSCULAR_RENDERER_H_INCLUDED
#define CUSTOM_WIDGET_CREPUSCULAR_RENDERER_H_INCLUDED

#include <gtf/scdr/GraphicsApi.h>
#include <gtf/scdr/GraphicsExecutionContext.h>
#include <gtf/scdr/RenderObjectUtils.h>
#include <gtf/scdr/ShaderProgram.h>
#include <gtf/scdr/VertexBuffer.h>
#include <gtf/scdr/WidgetRenderer.h>


struct CrepuscularGraphicObjects
{
    gtf::scdr::VertexBuffer*         vb;
    gtf::scdr::PipelineStateObject*  pso;
    gtf::scdr::ResourceBindingTable* rbt;
    gtf::scdr::SamplerState*         colorSampler;
    gtf::scdr::GraphicsApi::Value    driver;
};

class CrepuscularRenderObject;

struct CrepuscularConstantBufferBlock
{
    CrepuscularConstantBufferBlock()
        : color(gtf::scdr::SystemColors::Null)
        , xpos(0.f)
        , ypos(0.f)
        , decay(0.f)
        , density(0.f)
        , weight(0.f)
        , samples(0)
    {
        memset(object2Screen, 0, sizeof(object2Screen));
    }

    float            object2Screen[16];
    gtf::scdr::Color color;

    float            xpos;
    float            ypos;
    float            pad0[2];

    float            decay;
    float            density;
    float            weight;
    int              samples;
};

struct CrepuscularShaderResources
{
    gtf::scdr::ResourceBindingSlot constantBuffer;
    gtf::scdr::ResourceBindingSlot colorTexture;
    gtf::scdr::ResourceBindingSlot colorSampler;
};

struct CrepuscularProperties
{
    gtf::scdr::WidgetPropertiesCommon2D common2d;
    uint16_t                            color;
    uint16_t                            xpos;
    uint16_t                            ypos;
    uint16_t                            samples;
    uint16_t                            decay;
    uint16_t                            density;
    uint16_t                            weight;
};

struct CrepuscularFeatures
{
    gtf::scdr::WidgetFeaturesCommon2D common2d;
};

class CrepuscularRenderer : public gtf::scdr::WidgetRenderer
{
private:
    CrepuscularRenderer(const CrepuscularRenderer&);
    CrepuscularRenderer& operator=(const CrepuscularRenderer&);

    CrepuscularRenderer(const gtf::viewfactory::ViewFactory& factory_, gtf::scdr::GraphicsApi::Value graphicsApi_);

public:
    static gtf::scdr::WidgetRenderer* createInstance (
        const gtf::viewfactory::ViewFactory& factory_,
        gtf::scdr::GraphicsContext& context_,
        const void* createParams_);

    static void destroyInstance(
        gtf::scdr::WidgetRenderer* renderer_,
        gtf::scdr::GraphicsContext& context_);

    virtual ~CrepuscularRenderer();

    virtual bool init(gtf::scdr::GraphicsContext& context_);
    virtual void deinit(gtf::scdr::GraphicsContext& context_);

    virtual bool update(gtf::scdr::GraphicsContext& context_);
    virtual bool render(gtf::scdr::GraphicsContext& context_);

    virtual bool begin(gtf::scdr::GraphicsContext& context_);
    virtual bool end(gtf::scdr::GraphicsContext& context_);

    virtual bool traverse(
        gtf::widgetset::WidgetModel& widget_,
        const gtf::coreruntime::CoreContext& widgetContext_,
        const gtf::scdr::RenderObjectStackFrame& parentStackFrame_,
        gtf::scdr::RenderObjectStackFrame& objectStackFrame_,
        gtf::scdr::RenderObject& object_,
        gtf::scdr::TraverserContext& traverserContext_,
        gtf::scdr::GraphicsContext& graphicsContext_);

    virtual gtf::scdr::RenderObject* createRenderObject(
        const gtf::widgetset::WidgetModel& widget_,
        const gtf::coreruntime::CoreContext& widgetContext_,
        gtf::scdr::GraphicsContext& graphcisContext_);

    virtual void destroyRenderObject(
        gtf::scdr::RenderObject* object_,
        gtf::scdr::GraphicsContext& graphicsContext_);

    // Custom Render functions
    static bool doRender(void* userData_, gtf::scdr::GraphicsExecutionContext& context_);
    static bool doDelete(void* memory_, gtf::scdr::GraphicsExecutionContext& context_);

public:
    CrepuscularRenderObject*  objects;
    CrepuscularGraphicObjects graphicObjects;
    CrepuscularProperties     properties;
    CrepuscularFeatures       features;
};

#endif // CUSTOM_WIDGET_CREPUSCULAR_RENDERER_H_INCLUDED
