////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_WIDGET_CREPUSCULAR_EFFECT_D3D11_SHADER_H_INCLUDED
#define CUSTOM_WIDGET_CREPUSCULAR_EFFECT_D3D11_SHADER_H_INCLUDED

static const char* VertexShaderD3D11 =
"struct VS_IN                                                       "
"{                                                                  "
"    float2 pos : POSITION0;                                        "
"    float2 tex : TEXCOORD0;                                        "
"};                                                                 "
"                                                                   "
"struct VS_OUT                                                      "
"{                                                                  "
"    float4 pos : SV_POSITION0;                                     "
"    float2 tex : TEXCOORD0;                                        "
"};                                                                 "
"                                                                   "
"cbuffer cbEffectParams                                             "
"{                                                                  "
"    float4x4 object2World;                                         "
"    float4   color;                                                "
"    float2   pos;                                                  "
"    float2   pad0;                                                 "
"                                                                   "
"    float decay;                                                   "
"    float density;                                                 "
"    float weight;                                                  "
"    int   samples;                                                 "
"};                                                                 "
"                                                                   "
"VS_OUT main(VS_IN vsIn_)                                           "
"{                                                                  "
"    VS_OUT vsOut = (VS_OUT)0;                                      "
"                                                                   "
"    float4 tmp = float4(vsIn_.pos, 0, 1);                          "
"    vsOut.pos  = mul(tmp, object2World);                           "
"    vsOut.tex  = float2(vsIn_.tex.x, 1.f - vsIn_.tex.y);           "
"                                                                   "
"    return vsOut;                                                  "
"};                                                                 ";

static const char* PixelShaderD3D11 =
"struct PS_IN                                                       "
"{                                                                  "
"    float4 pos : SV_POSITION0;                                     "
"    float2 tex : TEXCOORD0;                                        "
"};                                                                 "
"                                                                   "
"struct PS_OUT                                                      "
"{                                                                  "
"    float4 c0 : SV_TARGET0;                                        "
"};                                                                 "
"                                                                   "
"Texture2D<float4> colorTexture;                                    "
"SamplerState      colorSampler;                                    "
"                                                                   "
"cbuffer cbEffectParams                                             "
"{                                                                  "
"    float4x4 object2World;                                         "
"    float4   lightColor;                                           "
"    float2   lightPos;                                             "
"    float2   pad0;                                                 "
"                                                                   "
"    float decay;                                                   "
"    float density;                                                 "
"    float weight;                                                  "
"    int   samples;                                                 "
"};                                                                 "
"                                                                   "
"PS_OUT main(PS_IN psIn_)                                           "
"{                                                                  "
"    PS_OUT psOut;                                                  "
"                                                                   "
"    int iSamples = samples;                                        "
"    float tDecay = pow(decay, rcp((float)iSamples));               "
"    float tWeight = weight;                                        "
"                                                                   "
"    float2 lpos = lightPos;                                        "
"    float2 tex = psIn_.tex;                                        "
"    float4 color = colorTexture.Sample(colorSampler, tex);         "
"    float2 dTex = (tex - lpos) * rcp(iSamples) * density;          "
"    float acc = color.w;                                           "
"    float idecay = 1.f;                                            "
"                                                                   "
"    for (int i = 0; i < iSamples; ++i)                             "
"    {                                                              "
"        tex -= dTex;                                               "
"        float sample = colorTexture.Sample(colorSampler, tex).w;   "
"        sample *= idecay * tWeight;                                "
"        acc += sample;                                             "
"        idecay *= tDecay;                                          "
"    }                                                              "
"                                                                   "
"    acc /= (float)iSamples;                                        "
"                                                                   "
"    color.xyz += lightColor.xyz;                                   "
"    color.w   += acc;                                              "
"                                                                   "
"    psOut.c0 = color;                                              "
"    return psOut;                                                  "
"};                                                                 ";

#endif
