////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#ifndef CUSTOM_EFFECT_DESCRIPTOR_H_INCLUDED
#define CUSTOM_EFFECT_DESCRIPTOR_H_INCLUDED

#include <gtf/metainfo/PropertyDescriptor.h>

#include <gtf/scdr/GraphicsApi.h>

typedef gtf::metainfo::PropertyDescriptor CustomEffectVariable;

struct CustomEffectShader
{
    struct
    {
        const char* vertex;
        const char* pixel;
    } d3d11;

    struct
    {
        const char* vertex;
        const char* pixel;
    } gles20;

    struct
    {
        const char* vertex;
        const char* pixel;
    } gles3;
};

struct CustomEffectTransparencyMode
{
    enum Value
    {
        Disabled,
        Default,
        Premultiplied
    };
};

struct CustomEffectDescriptor
{
    const char*                         name;
    CustomEffectShader                  shader;
    CustomEffectVariable*               variables;
    uint32_t                            variableCount;
    CustomEffectTransparencyMode::Value transparencyMode;
    bool                                isOffscreen;
};

struct CustomEffectCreateParams
{
    gtf::scdr::GraphicsApi::Value api;
    const CustomEffectDescriptor* descriptor;
};

struct CustomEffectDescriptorSet
{
    const CustomEffectDescriptor* effects;
    uint32_t                      count;
};

#endif
