////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <gtf/scdr/FramebufferClearFlags.h>

#include "CustomEffectUtils.h"

bool ClearOffscreenFramebuffer(void* customEffectRenderObject_, gtf::scdr::GraphicsExecutionContext& context_)
{
    GTF_UNUSED_PARAM(context_);

    static const float clearColor[4] = { 0.0, 0.0, 0.0, 0.0 };

    if (customEffectRenderObject_)
    {
        const uint32_t clearFlags =
            gtf::scdr::FramebufferClearFlags::Color |
            gtf::scdr::FramebufferClearFlags::Depth |
            gtf::scdr::FramebufferClearFlags::Stencil;

        CustomEffectRenderObject* const object = static_cast<CustomEffectRenderObject*>(customEffectRenderObject_);

        context_.clearFramebuffer(object->offscreenFb, clearFlags, clearColor, 1.f, 0);
    }

    return true;
}

bool DeleteRenderFunctionContext(void* memory_, gtf::scdr::GraphicsExecutionContext& context_)
{
    FREE_OBJECT(context_.getAllocator(gtf::scdr::AllocatorType::FrameAllocator), static_cast<RenderFunctionContext*>(memory_));
    return true;
}

void SetSamplerBinding(gtf::scdr::ResourceBindingDescriptor& desc_, const char* name_, uint32_t slot_)
{
    desc_.name         = name_;
    desc_.resourceType = gtf::scdr::ResourceType::Sampler;
    desc_.slotIndex    = slot_;
    desc_.offset       = 0;
}

void SetConstantBufferBinding(gtf::scdr::ResourceBindingDescriptor& desc_, const char* name_, uint32_t offset_)
{
    desc_.name         = name_;
    desc_.resourceType = gtf::scdr::ResourceType::ConstantBuffer;
    desc_.slotIndex    = 0;
    desc_.offset       = offset_;
}

void SetTexture2DBinding(gtf::scdr::ResourceBindingDescriptor& desc_, const char* name_, uint32_t slot_)
{
    desc_.name         = name_;
    desc_.resourceType = gtf::scdr::ResourceType::Texture2D;
    desc_.slotIndex    = slot_;
    desc_.offset       = 0;
}

void SetScalarUniformVariable(ScalarUniformVariable& var_, gtf::type::TypeId::typeId_t type_, uint16_t offset_, uint16_t handle_)
{
    var_.handle = handle_;
    var_.offset = offset_;
    var_.type   = type_;
}

void SetImageVariable(ImageVariable& var_, uint16_t slot_, uint16_t handle_)
{
    var_.handle = handle_;
    var_.slot   = slot_;
}

uint16_t ResolvePropertyHandle(const gtf::viewfactory::ViewFactory& factory_, const char* widgetName_, const char* propertyName_)
{
    gtf::viewfactory::ViewFactory::WidgetPropertyReference wpr =
    {
        { widgetName_, gtf::types::limits::uint16_max }, { propertyName_, gtf::types::limits::uint16_max }
    };

    factory_.resolveProperties(&wpr, 1);

    return wpr.widgetPropertyRef.ref;
}
