////////////////////////////////////////////////////////////////////////////////
// Copyright (c) Elektrobit Automotive GmbH
// Alle Rechte vorbehalten. All Rights Reserved.
//
// Information contained herein is subject to change without notice.
// Elektrobit retains ownership and all other rights in the software and each
// component thereof.
// Any reproduction of the software or components thereof without the prior
// written permission of Elektrobit is prohibited.
////////////////////////////////////////////////////////////////////////////////

#include <gtf/metainfo/PropertyDescriptor.h>

#include "CustomEffectDescriptor.h"
#include "CustomEffectRenderer.h"
#include "LoadedStageImpl.h"

extern const CustomEffectDescriptorSet* GetCustomEffectDescriptorSet();

customeffect::LoadedStageImpl::LoadedStageImpl(gtf::dependencyresolver::DependencyContainerHandle & container_)
    : container(container_)
    , effectDescriptors(NULL)
    , descCount(0)
{
    container->getAll<gtf::scdr::SDK>(sdkHandles);

    const CustomEffectDescriptorSet* const set = GetCustomEffectDescriptorSet();
    const uint32_t cnt                         = set->count;

    descCount         = cnt * sdkHandles.size();
    effectDescriptors = static_cast<CustomEffectCreateParams*>(gtf_mem_malloc(sizeof(CustomEffectCreateParams) * descCount));

    if (effectDescriptors)
    {
        CustomEffectCreateParams* p = effectDescriptors;
        for (gtf::container::Array<gtf::scdr::SDKHandle>::Iterator it = sdkHandles.begin(), e = sdkHandles.end(); it != e; ++it)
        {
            registerRenderer(*it, p);
            p += cnt;
        }
    }
}

customeffect::LoadedStageImpl::~LoadedStageImpl()
{
    CustomEffectCreateParams* p                = effectDescriptors;
    const CustomEffectDescriptorSet* const set = GetCustomEffectDescriptorSet();

    if (effectDescriptors)
    {
        for (gtf::container::Array<gtf::scdr::SDKHandle>::Iterator it = sdkHandles.begin(), e = sdkHandles.end(); it != e; ++it)
        {
            for (uint32_t i = 0; i < set->count; ++i)
            {
                (*it)->unregisterRenderer(p->descriptor->name);
                p++;
            }
        }
    }

    sdkHandles.clear();
    gtf_mem_free(effectDescriptors);
}

bool customeffect::LoadedStageImpl::registerRenderer(gtf::scdr::SDKHandle& sdk, CustomEffectCreateParams* params_)
{
    const CustomEffectDescriptorSet* const set = GetCustomEffectDescriptorSet();
    const uint32_t cnt                         = set->count;

    for (uint32_t i = 0; i < cnt; ++i)
    {
        params_->api        = sdk->getGraphicsApi();
        params_->descriptor = set->effects + i;

        gtf::scdr::WidgetRendererDescriptor wrdesc;

        wrdesc.className       = set->effects[i].name;
        wrdesc.createCallback  = CustomEffectRenderer::createInstance ;
        wrdesc.destroyCallback = CustomEffectRenderer::destroyInstance;
        wrdesc.createParams    = params_;

        params_++;

        if (!sdk->registerRenderer(wrdesc))
        {
            return false;
        }
    }

    return true;
}
